/****************************************************************************/
/**    GNOM - Gnom is NOt MCuT                                             **/
/**  Copyright (C) 1997 Jochen Sarrazin, DG6VJ. All righs reserved.        **/
/**                                                                        **/
/**  Globale Variablen, Datenstrukturen etc                                **/
/****************************************************************************/

#ifndef GLOBAL_H
#define GLOBAL_H


#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <fcntl.h>
#include <unistd.h>
#include <termios.h>
#include <syslog.h>
#include <signal.h>
#include <utime.h>
#include <sys/wait.h>

#ifdef SLANG
#include "slcombat.h"
#else
#include <ncurses.h>
#endif


#define MAJORLEVEL 0
#define MINORLEVEL 3
#define PATCHLEVEL 9
#define DATUM "07.09.97"

#define MAXCHAN 250			/* Maximale Kanal-Anzahl */
#define MAXFRAMELEN 256			/* Maximale Framelnge (Kernel-AX.25 macht 512! */
#define MONICHAN 0			/* Monitor-Kanal-Nummer */
#define VORSCHREIB_ANZCOLS 256		/* Wenn der Zeilenumbruch auf 0 steht, knnen maximal
                                	 * diese Anzahl Zeichen eingetippt werden. */

#ifndef TRUE
#define TRUE 1
#endif

#ifndef FALSE
#define FALSE 0
#endif




typedef unsigned char uchar;


typedef struct
{
  uint len;				/* Lnge des Files */
  uint bytes_got;			/* Anzahl schon empfangener Bytes */
  uchar calc_crc;			/* Mit CRC-Berechnung? */
  unsigned short crc_rx;		/* Empfangene Checksumme */
  unsigned short crc_tx;		/* Selbst errechnete Checksumme */
  unsigned long crc32;			/* 32-Bit-CRC */
  int fd;				/* FD des Files */
  int markfd;				/* Beim EBIN: Merken, welche Bytes schon empfangen wurden */
  char *name;				/* Filename */
  char *fullname;			/* den ganzen Filenamen mit Verzeichnissen */
  time_t starttime;			/* Wann hat die Sendung angefangen? */
  uchar type;				/* 1 = BIN, 2 = EBIN */
} s_autobin;

/* Verkettete Liste (Ringbuffer) fr die Scrollbuffer */
typedef struct s_scroll
{
  struct s_scroll *next;		/* Nchstes Listenfeld */
  struct s_scroll *last;		/* Letztes Listenfeld */
  char *text;				/* Text in dieser Zeile */
  char *attrib;				/* Nummer des Attributes im attribs-Array*/
  int blen;				/* Lnge aller Zeichen in diesem String */
} s_scroll;

/* Verkettete Liste (Ringbuffer) fr die Vorschreibpuffer */
typedef struct s_vorschreib
{
  struct s_vorschreib *next;		/* Nchstes Listenfeld */
  struct s_vorschreib *last;		/* Letztes Listenfeld */
  char text[VORSCHREIB_ANZCOLS+1];	/* Text in dieser Zeile */
  int len;				/* aktuelle Lnge dieser Zeile */
} s_vorschreib;

/*************************** SEND-QUEUE *****************************************/
typedef struct
{
  int len;
  char *text;
} s_plaintext;

typedef struct
{
  int fd;				/* File-Descriptor des Files. -1 wenn geschlossen */
  char *pfad;				/* Kompletten Pfad auf das File */
} s_plainfile;

typedef struct s_sendqueue
{
  struct s_sendqueue *next;		/* Zeiger aufs nchste Listenfeld */
  int type;				/* Was soll da gesendet werden?
                                         *    1 = Normaler Text
                                         *    2 = ein File 1:1 aussenden
                                         */
  s_plaintext *plaintext;		/* Normaler Text */
  s_plainfile *plainfile;
  s_autobin *autobin;
} s_sendqueue;
/********************************************************************************/


/************* Alle empfangenen Texte mssen erstmal hier durch *****************/
/************* (Umwandlung in einzelne Zeilen bei Linemode etc) *****************/
typedef struct s_rxframes
{
  char *text;
  int len;
} s_rxframes;
/********************************************************************************/


/************  Eine Liste der Rufzeichen, die wir bislang in einer  *************/
/************  *** connected to - Zeile empfangen haben  ************************/
typedef struct _s_conncalls
{
  struct _s_conncalls *next;
  char call[10];
} s_conncalls;
/********************************************************************************/

typedef struct
{
  int fd;
  char *filename;
  int local;				/* 1 = Sysop hat das File von der Konsole aus geffnet,
                                         * 2 = Ein User hat das File per Remote-Befehl geffnet. */
} s_saveplain;

typedef struct
{
  int pwmode;				/* Was fr ein Passwort? Siehe password.c */
  char *pw;				/* Bei String-Passwrtern das Passwort, bei DieBox der Dateiname */
  unsigned short rmnc_pw;		/* Die PW-Zahl beim RMNC */
  struct tm *dbtime;			/* Die DieBox-Loginzeit */
} s_pwinfo;

typedef struct
{
  int fd;
  char *name;
  int bytes_got;
  int len;
  time_t starttime;
} s_auto7plus;

typedef struct
{
  int save_it;				/* Ab jetzt speichern? */
  char *str;
  int len;				/* Lnge von str */
} s_ebin;

#define CH_REMOTE 1			/* Remote */
#define CH_DISC 2			/* Disconnecte den Link, wen alles gesendet wurde */
#define CH_PROMPT 4			/* Prompt nach //-Befehlen aussenden? */
#define CH_SAVE 8			/* Wird gerade ein Text gespeichert? */
#define CH_7PLUS 16			/* Wird gerade ein 7plus-File gespeichert? */
#define CH_AUTOBINRX 32			/* Wird gerade ein Autobin-File gespeichert? */
#define CH_NEWDATA 64			/* Auf diesem Kanal sind neue Daten angekommen */
#define CH_LINEMODE 128			/* Daten Zeilenweise oder Frameweise weitergeben? */
#define CH_CHECKCONN 256		/* Testet Connect-Meldungen */
#define CH_CHECKDISC 512		/* Testet Disconnect-Meldungen */
#define CH_AUTOBINTX 1024		/* Wird gerade ein Autobin-File gesendet? */
#define CH_CHECKAUTOBIN 2048		/* Nach AutoBIN gucken */
#define CH_CHECK7PLUS 4096		/* Nach 7plus gucken */
#define CH_COMP 8192			/* Online Huffman-Komprimierung */
#define CH_CAN_SEND 16384		/* Vertrgt der Treiber auf diesem Kanal noch ein Frame? */

#define TYPE_TERMINAL 1
#define TYPE_MAILBOX 2
#define TYPE_DIGI 3

/* Standard-Flags */
#define CH_STDFLAGS CH_REMOTE+CH_PROMPT+CH_LINEMODE+CH_CHECKDISC+CH_CHECKCONN+CH_CHECKAUTOBIN+CH_CHECK7PLUS
/* Informationen ber die einzelnen Kanle */
typedef struct
{
  char mycall[20];			/* Eigenes Rufzeichen auf diesem Kanal */
  char call[20];			/* Rufzeichen der connecteten Station */
  char firstcall[20];			/* auf der Frequenz direkt connectetes Call */
  char *digis;				/* ber welche Digipeater luft die Verbindung? */
  char *name;				/* Name der Gegenstation wenn bekannt, sonst NULL */
  uint flags;				/* Flags - siehe #define CH_* */
  uint oldflags;			/* Alte Flags */
  uchar type;				/* Was ist die Gegenstation fr ein Typ? */
  char port[50];			/* Portname */

  time_t logintime;			/* Wann hat sich die Station eingeloggt? */
  uchar status;				/* Status der Verbindung */
  uchar oldstatus;			/* Der alte Status */
  uchar consstat;			/* Status der Konsole. (AutoBIN-TX etc) */
  int convert_tx;			/* Konvertieren? Wenn ja -> in welchen Zeichensatz? */
  int convert_tx_old;			/* Alter Wert von convert */
  int convert_rx;			/* Konvertieren? Wenn ja -> in welchen Zeichensatz? */
  int convert_rx_old;			/* Alter Wert von convert */
  
  s_scroll *scroll_upper;		/* Zeiger auf das oberste Feld des Puffers */
  s_scroll *scroll_lower;		/* Zeiger auf das unterste Feld des Puffers */
  s_scroll *scroll_act;			/* Zeiger auf das Feld des Puffers, in das der nchste
                                	 * String geschrieben werden soll */
  s_scroll *scroll_screen;		/* Erste oben sichtbare Zeile */
  s_scroll *scroll_screen_lower;	/* Letzte sichtbare Zeile */
  s_scroll *scroll_screen_upper;	/* Oberste Zeile des Bildschirms, wenn ganz 'runter
                                         * gescrollt wurde */
  
  s_vorschreib *vor_upper;
  s_vorschreib *vor_lower;
  s_vorschreib *vor_act;		/* Zeiger auf das Feld, das im Augenblick bearbeitet wird */
  s_vorschreib *vor_screen;		/* Erste oben sichtbare Zeile */
  
  s_sendqueue *sendqueue;
  s_rxframes *rxframes;		 	/* Frames zum Umwandeln in den Linemode */
  s_conncalls *conncalls;		/* Eine Liste der Rufzeichen aller bisher empfangenen
                                         * Connect-Meldungen */
  s_saveplain *saveplain;		/* Struktur zum Speichern der RX-Daten */
  s_autobin *autobin;			/* Struktur zum AutoBINen */
  s_auto7plus *auto7plus;		/* Struktur fr die auto7plus-Geschichte */
  
  /* Fr die Passwort-Eingabe */
  s_pwinfo *pwinfo;			/* Fr die Passwort-Berechnung */
  
  /* Frs extended AutoBIN */
  s_ebin *ebin;
  
  char *AutoEBIN;
  int AutoEBINlen;
  unsigned long AutoEBINcrc;		/* 32-Bit-CRC */
  unsigned short AutoEBINretreq;	/* Anzahl retransmission-Requests */
  unsigned char AutoEBINblocksize;
  off_t AutoEBINoffset;
  int AutoEBINready;
  
  int x_akt;				/* Aktueller Cursor-Position im Vorschreib-Puffer */
  int y_akt;
  int x_screen;				/* Welches Zeichen ist ganz links auf dem Schirm zu sehen? */
  int umbruch;				/* Automatischer Zeilenumbruch nach X Bytes */
  
  unsigned long rx_bytes;		/* Anzahl empfangener Bytes */
  unsigned long tx_bytes;		/* Anzahl gesendeter Bytes */
  
  int tries;
  int unack;
  int squeue;
  
  int qsonr;
  int driverfd;
} s_chaninfo;

s_chaninfo *chaninfo[MAXCHAN+1];



/****************************************************************************/
/**                  Konfigurations-Informationen                          **/
typedef struct
{
  char *con;				/* Beim Connecten */
  char *disc;				/* Beim Disconnecten */
  uchar bell;				/* TRUE, wenn gebimmelt werden soll, sonst FALSE */
  uchar ringok;				/* //RING-Klingel einschalten? */
  char *ring;				/* Sound beim //RINGen */
  int klack;				/* Bei jedem empfangenen Frame klacken? */
} s_tones;

/* diverse standard-away-texte */
typedef struct
{
  char *weg;				/* Ich bin mal weg */
  char *da;				/* Ich bin wieder da */
  char *telefon;			/* Ich bin mal teflonieren */
  char *sleep;				/* Ich bin pennen */
} s_away_texte;

typedef struct
{
  s_tones *tones;			/* Bimmel-Tne beim Connecten */
  s_away_texte *awaytxt;		/* diverse away-texte */
  char mycall[10];			/* Eigenes Standard-Rufzeichen */
  char maindir[200];			/* Hauptverzeichnis, unter dem alles andere ist. */
  char *driver_sockets;			/* Eine Liste der sockets der einzelnen Hardware-Treiber */
  char stdport[50];
  int tabsize;				/* Anzahl Bytes, die einem Tab entsprechen */
  int clock;				/* Soll eine Uhr angezeigt werden? */
  int width;				/* Standard-Zeilenumbruch nach wievielen Bytes (Vorschreib)? */
} s_config;
s_config *config;
/****************************************************************************/


/* Wenn der SysOp sich als away eingetragen hat, steht hier der Text */
char *awaytext;
time_t awaytime;

/* Informationen aus diversen Config-Files */
time_t starttime;			/* Wann wurde dieser GNOM-Prozess gestartet? */
int framesize;				/* In dieser Gre werden die Frames zum Pr-Iface gesendet */
int maxframe;				/* Soviele Frames werden auf einmal zum PR-Interface gesendet */
int chancount;				/* Wieviele Kanle sollen benutzt werden knnen? */
int maxscrolllines;			/* Maximale Anzahl der scrollbaren Zeilen pro Kanal */
int maxvorschreiblines;			/* Maximale Anzahl an Vorschreib-Zeilen pro Kanal */
int pos_statusline;			/* Die Position der unteren Statuszeile (zwischen RX- und TX-Fenster) */
int rx_upper,rx_lower;			/* Obere- und Untere Zeile im RX-Fenster */
int tx_upper,tx_lower;			/* Obere- und Untere Zeile im TX-Fenster */
int size_tx;				/* Gre des TX-Fensters (in Zeilen) */
int size_rx;				/* Gre des RX-Fensters (in Zeilen) */
int size_col;				/* Anzahl Spalten auf dem Bildschirm */
int size_lines;				/* Anzahl Zeilen auf dem Schirm */
int colors[15];				/* Eingestellte Farben */
int attribs[11];			/* und die dazu gehrigen Attribute */

int ConsoleStatus;			/* Status der Console */
int AktChan;				/* Aktueller Kanal */
int x_akt;
int y_akt;
int x_screen;

/* Kernel-AX.25-Interface */
int insert;				/* Einfgen (1) oder berschreiben (0) */
int maxfd;

/* Keypad benutzen? */
int use_keypad;

/* Variablen fr die Fenster-Geschichte (ncurses) */
WINDOW *tx_win;
WINDOW *rx_win;
WINDOW *stoben_win;
WINDOW *stunten_win;




/* Defines fr die Attribut-Felder */
#define ATTR_STATUSLN 0
#define ATTR_STATUSLN_TRANS 10
#define ATTR_RX_SEND 1
#define ATTR_RX_EMPF 3
#define ATTR_RX_STATUS 4
#define ATTR_MON_PORTNAME 5
#define ATTR_MON_PROTOCOL 6
#define ATTR_MON_ADDR 7
#define ATTR_MON_DATA 8
#define ATTR_MON_AXHDR 9



/* Eine verkettete Liste fr die installierten Ports */
typedef struct _s_ports
{
  struct _s_ports *next;
  char *port;				/* Portname */
  char *desc;				/* Beschreibung */
  char *dev;				/* Device-Name */
  int baud;				/* Baudrate */
  uchar listen;				/* Lauschen wir auf diesem Port? */
  char *call;				/* Rufzeichen dieses Ports */
} s_ports;

s_ports *portsroot;


#endif

